/* ---------------------------------------------------------- */
/*  kome_display_drv.c                                        */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		V850/CPLD CRTC display driver
    @author		rerofumi
    @since		Jul.23.2007
*/
/*-----------------------------------------------------
 Copyright (C) 2007 rerofumi <rero2@yuumu.org>
 -  Kome wo kame.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "kome_display_drv.h"
#include "mono_ascii.h"

#ifdef __SDL__
#include "SDL.h"
#include "SDL_main.h"
#endif

/*-------------------------------*/
/* define                        */
/*-------------------------------*/


/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

//! `pPbgobt@
KDisplayPacket  display_packet[KOMEDISPLAY_PACKETSIZE];

//! pPbgNGXg
int  packet_num;
int  packet_ptr_start, packet_ptr_end;

//! CRTC \XCb`
int  crtc_sw;


//! SDLŕ\XN[
SDL_Surface *Screen;
//! SDLVSync Wait ǂ
int BeforeTiming;

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

//! pPbg1\
void  put_packet(KDisplayPacket *packet);

/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/*-------------------------------*/
/* implementation                */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- komeDisplayInitialize() */
/**
 * Display hCȍ
 * |[gݒ
 * 
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayInitialize(void)
{
  int  r;

  crtc_sw = 0;


  /* --- SDLŃCjVCY */
  r = SDL_Init(SDL_INIT_VIDEO |
	       SDL_INIT_AUDIO | 
	       SDL_INIT_TIMER |
	       SDL_INIT_JOYSTICK);
  if (r != 0) {
    /* -- SDLłȂ */
    printf("SDL initialize error.\n");
    return(DISPLAY_ERROR);
  }
  /* XN[ */
  Screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT,
                            SCREEN_DEPTH,
                            SDL_SWSURFACE);
  SDL_FillRect(Screen, 0, 0xff0000c0);
  /* XN[\ */
  SDL_UpdateRect(Screen, 0, 0, 0, 0);
  /* wait timer */
  BeforeTiming = SDL_GetTicks();
 
  return(DISPLAY_DONE);
}


/* --- komeDisplayFree() */
/**
 * Display hCoI
 * 
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayFree(void)
{
  SDL_FreeSurface(Screen);

  return(DISPLAY_DONE);
}


/* --- komeDisplayClearPacket() */
/**
 * Display packet ̃NA
 * 
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayClearPacket(void)
{
  packet_num = 0;
  packet_ptr_start = packet_ptr_end = 0;

  return(DISPLAY_DONE);
}



/* --- komeDisplayGetNumPacket() */
/**
 * Display packet ̃NGXg
 *
 * @param num [out] ݂̃NGXgpPbgԂ
 * 
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayGetNumPacket(int *num)
{
  *num = packet_num;

  return(DISPLAY_DONE);
}



/* --- komeDisplaySwCrtc() */
/**
 * CRTC ̕\XCb`
 *
 * @param sw [in] CRTC\Rg[
 * 
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplaySwCrtc(int sw)
{
  crtc_sw = sw;
  /* --- CRTC control */

  return(DISPLAY_DONE);
}



/* --- komeDisplayVSync() */
/**
 * VSync ҂\NGXg
 *
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayVSync(void)
{
  int  wait;

  /* --- CRTC mode check */
  if (crtc_sw == 0) {
    return(DISPLAY_NOT_ENABLE);
  }

  /* --- VBlank ҂ */
  wait = SDL_GetTicks() - BeforeTiming;
  if ((wait > 0) && (wait < 16)) {
    SDL_Delay(16-wait);
  }
  BeforeTiming = SDL_GetTicks();

  /* --- VBlankȂpPbg\ */
  while(packet_num > 0) {
    /* - VBlank check */
    /* === */
    /* - Packet display */
    put_packet(&(display_packet[packet_ptr_start]));
    packet_ptr_start = (packet_ptr_start + 1) % KOMEDISPLAY_PACKETSIZE;
    packet_num -= 1;
  }

  /* XN[\ */
  SDL_UpdateRect(Screen, 0, 0, 0, 0);

  return(DISPLAY_DONE);
}


/* --- komeDisplayASync() */
/**
 * ASynchronous \NGXg(CRTC OFFp)
 *
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayASync(void)
{
  while(packet_num > 0) {
    put_packet(&(display_packet[packet_ptr_start]));
    packet_ptr_start = (packet_ptr_start + 1) % KOMEDISPLAY_PACKETSIZE;
    packet_num -= 1;
  }

  /* XN[\ */
  SDL_UpdateRect(Screen, 0, 0, 0, 0);

  return(DISPLAY_DONE);
}



/* --- komeDisplayFill() */
/**
 * `tBNGXg
 *
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param w [in] \TCY w
 * @param h [in] \TCY h
 * @param fill [in] tB鐔l(0-255)
 *
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayFill(int x, int y, int w, int h, unsigned char fill)
{
  int  ptr;

  /* --- packet ptr */
  ptr = (packet_ptr_end + 1) % KOMEDISPLAY_PACKETSIZE;
  if (ptr == packet_ptr_start) {
    /*  packet full */
    return(DISPLAY_PACKET_FULL);
  }
  packet_num += 1;
  /* --- packet */
  display_packet[packet_ptr_end].data = 0;
  display_packet[packet_ptr_end].pos_x = x;
  display_packet[packet_ptr_end].pos_y = y;
  display_packet[packet_ptr_end].width = w;
  display_packet[packet_ptr_end].height = h;
  display_packet[packet_ptr_end].fill_data = fill;
  packet_ptr_end = ptr;

  return(DISPLAY_DONE);
}


/* --- komeDisplayClearScreen() */
/**
 * SʃNA
 * 0  fill ē]Ɗ҂pbNɂ
 *
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayClearScreen(void)
{
  komeDisplayClearPacket();
  komeDisplayFill(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0);
  if (crtc_sw == 0) {
    komeDisplayASync();
  }
  else {
    komeDisplayVSync();
  }

  return(DISPLAY_DONE);
}


/* --- komeDisplayPattern() */
/**
 * `p^[]NGXg
 *
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param w [in] \TCY w
 * @param h [in] \TCY h
 * @param data [in] \p^[f[^|C^
 *
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayPattern(int x, int y, int w, int h, const unsigned char *data)
{
  int  ptr;

  /* --- packet ptr */
  ptr = (packet_ptr_end + 1) % KOMEDISPLAY_PACKETSIZE;
  if (ptr == packet_ptr_start) {
    /*  packet full */
    return(DISPLAY_PACKET_FULL);
  }
  packet_num += 1;
  /* --- packet */
  display_packet[packet_ptr_end].data = (unsigned char *)data;
  display_packet[packet_ptr_end].pos_x = x;
  display_packet[packet_ptr_end].pos_y = y;
  display_packet[packet_ptr_end].width = w;
  display_packet[packet_ptr_end].height = h;
  display_packet[packet_ptr_end].fill_data = 0;
  packet_ptr_end = ptr;

  return(DISPLAY_DONE);
}



/* --- komeDisplayFont() */
/**
 * tHgp^[1\
 *
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param code [in] \LN^[
 *
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayFont(int x, int y, int code)
{
  int  ptr;

  /* --- assert */
  if ((code < 0) || (code > 127)) {
    return(DISPLAY_ERROR);
  }

  /* --- packet ptr */
  ptr = (packet_ptr_end + 1) % KOMEDISPLAY_PACKETSIZE;
  if (ptr == packet_ptr_start) {
    /*  packet full */
    return(DISPLAY_PACKET_FULL);
  }
  packet_num += 1;
  /* --- packet */
  display_packet[packet_ptr_end].data = (unsigned char *)ascii_table[code];
  display_packet[packet_ptr_end].pos_x = x;
  display_packet[packet_ptr_end].pos_y = y;
  display_packet[packet_ptr_end].width = 8;
  display_packet[packet_ptr_end].height = 8;
  display_packet[packet_ptr_end].fill_data = 0;
  packet_ptr_end = ptr;

  return(DISPLAY_DONE);
}


/* --- komeDisplayString() */
/**
 * tHgp^[XgO\
 *
 * @param x [in] \ʒu x
 * @param y [in] \ʒu y
 * @param str [in] \镶|C^
 *
 * @return Ȃ DISPLAY_DONEAȊOŎs
 *
 */
int komeDisplayString(int x, int y, char *str)
{
  int  pos_x, pos_y, safe;
  char *chr;

  chr = str;
  pos_x = x;
  pos_y = y;
  safe = 1000;
  while((*chr != 0x00) && (safe > 0)) {
    komeDisplayFont(pos_x, pos_y, *chr);
    chr += 1;
    safe -= 1;
    pos_x += 8;
    if (pos_x > (SCREEN_WIDTH-8)) {
      pos_x = 0;
      pos_y += 8;
      if (pos_y > (SCREEN_HEIGHT-8)) {
	break;
      }
    }
  }
  
  return(DISPLAY_DONE);
}




/*------------------------------------------------------*/
/* Interrupt                                            */
/*------------------------------------------------------*/
/*------------------------------------------------------*/
/* Local routine                                        */
/*------------------------------------------------------*/

//! pPbg1\
void  put_packet(KDisplayPacket *packet)
{
  int  loop_x, loop_y;
  unsigned long  *buf, pixel;
  unsigned char  *dat;

  //SDL_LockSurface(Screen);
  
  dat = packet->data;

  if (dat == 0) {
    pixel = (unsigned long)packet->fill_data;
    pixel = pixel | (pixel << 8) | (pixel << 16) | 0xff000000;
    for(loop_y=0; loop_y<packet->height; loop_y++) {
      buf = packet->pos_x + ((packet->pos_y + loop_y) * SCREEN_WIDTH) + (unsigned long *)Screen->pixels;
      for(loop_x=0; loop_x<packet->width; loop_x++) {
        *buf = pixel;
        buf += 1;
      }
    }
  }
  else {
    for(loop_y=0; loop_y<packet->height; loop_y++) {
      buf = packet->pos_x + ((packet->pos_y + loop_y) * SCREEN_WIDTH) + (unsigned long *)Screen->pixels;
      for(loop_x=0; loop_x<packet->width; loop_x++) {
        pixel = (unsigned long)*dat;
        pixel = pixel | (pixel << 8) | (pixel << 16) | 0xff000000;
        *buf = pixel;
        buf += 1;
        dat += 1;
      }
    }
  }

  SDL_UnlockSurface(Screen);
}

