/* ---------------------------------------------------------- */
/*  kv_shot.c                                                 */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
@brief		KomeVader enemy and own shot manage
@author		rerofumi
@since		Aug.05.2007
*/
/*-----------------------------------------------------
Copyright (C) 2007 rerofumi <rero2@yuumu.org>
-  Kome wo kame.
------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "kome_display_drv.h"
#include "kv_display.h"
#include "kv_sound.h"
#include "kv_myship.h"
#include "kv_enemy.h"
#include "kv_mystery.h"
#include "kv_shot.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

#define SHOT_SIZE_W  3
#define SHOT_SIZE_H  8

/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

//! eIuWFNg\
typedef struct {
	short  pos_x, pos_y;
	char   flag;
	char   timer;
} KVShot, PKVShot;


/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

//! @Vbg
KVShot  m_myshot;

//! @e̔ːEFCg
int  m_myshot_wait;

//! GU
KVShot  m_enemyshot[SHOT_ENEMY_SHOT_MAX];

//! Gě`ςJE^
int  m_enemyshot_counter;

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- kvShotInitialize() */
/**
* eǗ
* 
*/
void kvShotInitialize(void)
{
	int  i;

	m_myshot.flag = 0;
	for(i=0; i<SHOT_ENEMY_SHOT_MAX; i++) {
		m_enemyshot[i].flag = 0;
	}
	m_enemyshot_counter = 0;
	m_myshot_wait = 0;
}


/* --- kvShotMyshotState() */
/**
* @ȅԎ擾
* 
* @return SHOT_MOVINGŔ˒ASHOT_DONEŔˉ\
*
*/
int kvShotMyshotState(void)
{
	if (m_myshot.flag == 1) {
		return(SHOT_MOVING);
	}
	return(SHOT_DONE);
}


/* --- kvShotMyshotRequest() */
/**
* @e̔˃NGXg
* W͒e̐[
* 
* @return SHOT_DONEŔˁAݔ˒ŋۂSHOT_FULL
*
*/
int kvShotMyshotRequest(int x, int y)
{
	if (m_myshot.flag == 1) {
		return(SHOT_FULL);
	}

	m_myshot.pos_x = x;
	m_myshot.pos_y = y;
	m_myshot.flag = 1;
	m_myshot.timer = 0;
	kvSoundRequest(0, SE_SHOT);
	kvMysteryAddCount();

	return(SHOT_DONE);
}


/* --- kvShotEnemyshotState() */
/**
* GȅԎ擾
* 
* @return SHOT_FULLŔ˕sASHOT_DONEŔˉ\
*
*/
int kvShotEnemyshotState(void)
{
	int  i, empty;

	empty = 0;
	for(i=0; i<SHOT_ENEMY_SHOT_MAX; i++) {
		if (m_enemyshot[i].flag == 0) {
			empty = 1;
		}
	}
	if (empty == 0) {
		return(SHOT_FULL);
	}
	return(SHOT_DONE);
}


/* --- kvShotEnemyshotRequest() */
/**
* Ge̔˃NGXg
* 
* @return SHOT_DONEŔˁAݔ˒ŋۂSHOT_FULL
*
*/
int kvShotEnemyshotRequest(int x, int y)
{
	int  i, empty;

	empty = 0;
	for(i=0; i<SHOT_ENEMY_SHOT_MAX; i++) {
		if (m_enemyshot[i].flag == 0) {
			m_enemyshot[i].pos_x = x;
			m_enemyshot[i].pos_y = y;
			m_enemyshot[i].flag = (m_enemyshot_counter % 2) + 1;
			m_enemyshot[i].timer = 0;
			empty = 1;
			break;
		}
	}
	if (empty == 0) {
		return(SHOT_FULL);
	}
	m_enemyshot_counter += 1;
	return(SHOT_DONE);
}


/* --- kvShotTick() */
/**
* ëړՓˊǗ
* 
* @return G|Ƃ̉Z|Cg
*/
int kvShotTick(void)
{
	int  hit;
	int  point, result;
	int  i;

	point = 0;

	/* --- @e */
	if (m_myshot_wait > 0) {
		m_myshot_wait -= 1;
	}
	else if (m_myshot.flag != 0) {
		hit = 0;
		/* Öʒu */
		kvDisplayShot(m_myshot.pos_x-1, m_myshot.pos_y, DISPLAY_SHOT_ERASE);
		/* ړ */
		m_myshot.pos_y -= 4;
		/* g[`JƂ̏ՓˊmF */
		if (kvMyshipShieldCheckHit(m_myshot.pos_x, m_myshot.pos_y) != MYSHIP_DONE) {
			hit = 1;
			m_myshot.flag = 0;
		}
		/* GƂ̏ՓˊmF */
		result = kvEnemyCheckHit(m_myshot.pos_x, m_myshot.pos_y);
		if (result != ENEMY_DONE) {
			kvSoundRequest(1, SE_INVADER_HIT);
			hit = 1;
			m_myshot.flag = 0;
			m_myshot_wait = 20;
			switch(result) {
			case ENEMY_HIT_10:
				point += 10;
				break;
			case ENEMY_HIT_20:
				point += 20;
				break;
			case ENEMY_HIT_30:
				point += 30;
				break;
			default:
				break;
			}
		}
		/* ~XeƂ̏ՓˊmF */
		result = kvMysteryCheckHit(m_myshot.pos_x, m_myshot.pos_y);
		if (result == MYSTERY_HIT) {
			kvSoundRequest(3, SE_MYSTERY_HIT);
			point += kvMysteryGetScore();
			hit = 1;
			m_myshot.flag = 0;
		}
		/* U */
		if (m_myshot.pos_y < 8) {
			hit = 1;
			m_myshot.flag = 0;
		}
		/* Vʒuŏ */
		if (hit == 0) {
			kvDisplayShot(m_myshot.pos_x-1, m_myshot.pos_y, DISPLAY_SHOT_MYSHIP);
		}
	}

	/* --- Ge */
	m_enemyshot_counter += 1;
	for(i=0; i<SHOT_ENEMY_SHOT_MAX; i++) {
		hit = 0;
		if (m_enemyshot[i].flag != 0) {
			/* Öʒu */
			kvDisplayShot(m_enemyshot[i].pos_x-1,
				m_enemyshot[i].pos_y,
				DISPLAY_SHOT_ERASE);
			/* ړ */
			m_enemyshot[i].pos_y += 3;
			m_enemyshot[i].timer += 1;
			/* g[`JƂ̏Փ˔ */
			if (kvMyshipShieldCheckHit(m_enemyshot[i].pos_x,
				m_enemyshot[i].pos_y+8) != MYSHIP_DONE) {
					hit = 1;
					m_enemyshot[i].flag = 0;
			}
			/* @Ƃ̏Փ˔ */
			if (kvMyshipCheckHit(m_enemyshot[i].pos_x,
				m_enemyshot[i].pos_y+8) != MYSHIP_DONE) {
					hit = 1;
					m_enemyshot[i].flag = 0;
			}
			/* U */
			if (m_enemyshot[i].pos_y > (SCREEN_HEIGHT-(240-216)-1)) {
				hit = 1;
				m_enemyshot[i].flag = 0;
			}
			/* Vʒuŏ */
			if (hit == 0) {
				if (m_enemyshot[i].flag == 1) {
					kvDisplayShot(m_enemyshot[i].pos_x-1,
						m_enemyshot[i].pos_y,
						DISPLAY_SHOT_ENEMY_1_1 + (m_enemyshot[i].timer % 2));
				}
				else {
					kvDisplayShot(m_enemyshot[i].pos_x-1,
						m_enemyshot[i].pos_y,
						DISPLAY_SHOT_ENEMY_2_1 + (m_enemyshot[i].timer % 2));
				}
			}
		}
	}

	return(point);
}

